import os
import shutil

import langcodes
import requests

import unidata_blocks

blocks_doc_url = 'https://www.unicode.org/Public/UNIDATA/Blocks.txt'

project_root_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
unidata_dir = os.path.join(project_root_dir, 'src', 'unidata_blocks', 'unidata')
blocks_file_name = 'Blocks.txt'
blocks_file_path = os.path.join(unidata_dir, blocks_file_name)
translations_dir = os.path.join(unidata_dir, 'translations')
translations_tmp_dir = os.path.join(project_root_dir, 'build', 'tmp', 'translations')
lang_codes_file_path = os.path.join(unidata_dir, 'lang-codes.txt')


def main():
    response = requests.get(blocks_doc_url)
    assert response.ok
    assert 'text/plain' in response.headers['Content-Type']
    with open(blocks_file_path, 'w', encoding='utf-8') as file:
        file.write(response.text)
    unicode_version, blocks = unidata_blocks._parse_blocks(response.text)

    if os.path.exists(translations_tmp_dir):
        shutil.rmtree(translations_tmp_dir)
    os.makedirs(translations_tmp_dir)

    lang_codes = ['en']

    for lang_file_name in os.listdir(translations_dir):
        if not lang_file_name.endswith('.txt'):
            continue
        lang_code = langcodes.standardize_tag(lang_file_name.removesuffix('.txt'))
        assert f'{lang_code.lower()}.txt' == lang_file_name, f"Illegal file name: '{lang_file_name}'"
        lang_codes.append(lang_code)

        lang_file_path = os.path.join(translations_dir, lang_file_name)
        with open(lang_file_path, 'r', encoding='utf-8') as file:
            translations = unidata_blocks._parse_translations(file.read())

        lang_tmp_file_path = os.path.join(translations_tmp_dir, lang_file_name)
        with open(lang_tmp_file_path, 'w', encoding='utf-8') as file:
            file.write(f'# Unicode: {unicode_version}\n')
            file.write(f'# {lang_code}\n\n')
            for block in blocks:
                localized_name = translations.get(block.name, None)
                if localized_name is None:
                    file.write(f'# TODO # {block.name}:\n')
                else:
                    file.write(f'{block.name}: {localized_name}\n')
    shutil.rmtree(translations_dir)
    os.rename(translations_tmp_dir, translations_dir)

    lang_codes.sort()
    with open(lang_codes_file_path, 'w', encoding='utf-8') as file:
        file.write('\n'.join(lang_codes))
        file.write('\n')


if __name__ == '__main__':
    main()



import pkgutil
import re
from typing import Final

import langcodes


def _load_data_text(resource: str) -> str:
    return pkgutil.get_data(__package__, resource).decode(encoding='utf-8')


def _parse_blocks(text: str) -> tuple[str, list['UnicodeBlock']]:
    blocks = []
    lines = text.split('\n')
    version = lines[0].removeprefix('# Blocks-').removesuffix('.txt')
    for line in lines:
        line = line.split('#', 1)[0].strip()
        if line == '':
            continue
        tokens = re.split(r'\.\.|;\s', line)
        code_start = int(tokens[0], 16)
        code_end = int(tokens[1], 16)
        name = tokens[2]
        blocks.append(UnicodeBlock(code_start, code_end, name))
    return version, blocks


def _parse_lang_codes(text: str) -> list[str]:
    lang_codes = []
    lines = text.split('\n')
    for line in lines:
        line = line.strip()
        if line == '':
            continue
        lang_codes.append(line)
    return lang_codes


def _parse_translations(text: str) -> dict[str, str]:
    translations = {}
    lines = text.split('\n')
    for line in lines:
        line = line.split('#', 1)[0].strip()
        if line == '':
            continue
        tokens = line.split(':')
        if len(tokens) == 2:
            translations[tokens[0].strip()] = tokens[1].strip()
    return translations


class UnicodeBlock:
    _supported_languages: Final[list[str]] = _parse_lang_codes(_load_data_text('unidata/lang-codes.txt'))
    _translations_registry: Final[dict[str, dict[str, str]]] = {}

    def __init__(self, code_start: int, code_end: int, name: str):
        self.code_start = code_start
        self.code_end = code_end
        self.name = name
        self.capacity = code_end - code_start + 1
        self.printable_count = 0
        for code_point in range(code_start, code_end + 1):
            c = chr(code_point)
            if c.isprintable():
                self.printable_count += 1

    def __str__(self):
        return f'{self.code_start:04X}..{self.code_end:04X}; {self.name}'

    def name_localized(self, lang_code: str, __default: str = None) -> str | None:
        closest_language = langcodes.closest_supported_match(lang_code, UnicodeBlock._supported_languages)
        if closest_language is None:
            return __default
        if closest_language == 'en':
            return self.name
        if closest_language in UnicodeBlock._translations_registry:
            translations = UnicodeBlock._translations_registry[closest_language]
        else:
            translations = _parse_translations(_load_data_text(f'unidata/translations/{closest_language.lower()}.txt'))
            UnicodeBlock._translations_registry[closest_language] = translations
        return translations.get(self.name, __default)


def _standardize_block_name(name: str) -> str:
    return name.strip().lower().replace('-', ' ').replace('_', ' ')


unicode_version, _blocks = _parse_blocks(_load_data_text('unidata/Blocks.txt'))
_name_to_block = {_standardize_block_name(block.name): block for block in _blocks}


def get_block_by_code_point(code_point: int) -> UnicodeBlock | None:
    if _blocks[0].code_start <= code_point <= _blocks[-1].code_end:
        for block in _blocks:
            if block.code_start <= code_point <= block.code_end:
                return block
    return None


def get_block_by_name(name: str) -> UnicodeBlock | None:
    return _name_to_block.get(_standardize_block_name(name), None)


def get_block_by_chr(c: str) -> UnicodeBlock | None:
    return get_block_by_code_point(ord(c))


def get_blocks() -> list[UnicodeBlock]:
    return list(_blocks)



import unidata_blocks


def test_unicode_version():
    assert unidata_blocks.unicode_version == '15.0.0'


def test_no_block():
    assert unidata_blocks.get_block_by_code_point(-1) is None
    assert unidata_blocks.get_block_by_code_point(0x999999) is None


def test_code_point_query():
    block = unidata_blocks.get_block_by_code_point(30)
    assert block.code_start == 0x0000
    assert block.code_end == 0x007F
    assert block.name == 'Basic Latin'
    assert block.capacity == 128
    assert block.printable_count == 95

    block = unidata_blocks.get_block_by_code_point(130)
    assert block.code_start == 0x0080
    assert block.code_end == 0xFF
    assert block.name == 'Latin-1 Supplement'
    assert block.capacity == 128
    assert block.printable_count == 94


def test_name_query():
    block = unidata_blocks.get_block_by_name('CJK Unified Ideographs')
    assert block.code_start == 0x4E00
    assert block.code_end == 0x9FFF
    assert block.name == 'CJK Unified Ideographs'
    assert block == unidata_blocks.get_block_by_name('CJK-Unified-Ideographs')
    assert block == unidata_blocks.get_block_by_name('CJK_Unified_Ideographs')
    assert block == unidata_blocks.get_block_by_name('cjk unified ideographs')
    assert block == unidata_blocks.get_block_by_name('CJK UNIFIED IDEOGRAPHS')


def test_chr_query():
    block = unidata_blocks.get_block_by_chr('A')
    assert block.code_start == 0x0000
    assert block.code_end == 0x007F
    assert block.name == 'Basic Latin'
    assert block == unidata_blocks.get_block_by_chr('B')

    block = unidata_blocks.get_block_by_chr('汉')
    assert block.code_start == 0x4E00
    assert block.code_end == 0x9FFF
    assert block.name == 'CJK Unified Ideographs'
    assert block == unidata_blocks.get_block_by_chr('字')


def test_all_query():
    blocks = unidata_blocks.get_blocks()
    assert len(blocks) > 0
    assert blocks[0].name == 'Basic Latin'


def test_to_str():
    block = unidata_blocks.get_block_by_code_point(0x0000)
    assert str(block) == '0000..007F; Basic Latin'

    block = unidata_blocks.get_block_by_code_point(0x4E00)
    assert str(block) == '4E00..9FFF; CJK Unified Ideographs'

    block = unidata_blocks.get_block_by_code_point(0xF0000)
    assert str(block) == 'F0000..FFFFF; Supplementary Private Use Area-A'

    block = unidata_blocks.get_block_by_code_point(0x100000)
    assert str(block) == '100000..10FFFF; Supplementary Private Use Area-B'


def test_i18n():
    block = unidata_blocks.get_block_by_code_point(0x0000)
    assert block.name_localized('en') == 'Basic Latin'
    assert block.name_localized('EN') == 'Basic Latin'
    assert block.name_localized('zh') == '基本拉丁'
    assert block.name_localized('ZH') == '基本拉丁'
    assert block.name_localized('zh-hans') == '基本拉丁'
    assert block.name_localized('zh-chs') == '基本拉丁'
    assert block.name_localized('zh-cn') == '基本拉丁'
    assert block.name_localized('no-language') is None
    assert block.name_localized('no-language', 'abc') == 'abc'
